
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::SubmissionsController < ApplicationController
  include ::Paginator

  before_action :set_new_submission, only: :index
  before_action :set_submission, only: [:show, :answer, :destroy]
  before_action :authenticate_user!
  before_action :authorize!, except: :create

  def index
    submissions = paginate Submission.all
    render json: submissions
  end

  def show
    render json: @submission
  end

  def create
    learning_object = LearningObject.where(id: submission_params[:learning_object_id]).first
    if learning_object.blank?
      render :json, status: :not_found
    else
      @submission = Submission.new(learning_object: learning_object, submitter: current_user, status: :submitted)

      authorize @submission

      if @submission.save
        assign_curators
        @submission.learning_object.submitted!
        render json: @submission, status: :created
      else
        render json: @submission.errors, status: :unprocessable_entity
      end
    end
  end

  def answer
    @submission.justification = answer_params[:justification]
    answers = []
    questions = Question.active.pluck(:id)
    answers = answer_params[:answers]
    if questions.size > answers.size
      render json: { "error": "Active questions missing" }, status: :unprocessable_entity
    else
      answers.each do |answer|
        if !questions.include?(answer[:question_id])
          render json: {"error": "Inactive questions not permitted"}, status: :unprocessable_entity
          return
        else
          @submission.answers << Answer.create(answer)
        end
      end
      @submission.curator = current_user
      if @submission.save
        if @submission.accepted?
          publisher = LearningObjectPublisher.new(DspaceService.create_client)
          publisher.publish @submission.learning_object
        end
        render json: @submission, status: :ok
      else
        render json: @submission.errors, status: :unprocessable_entity
      end
    end
  end

  def destroy
    @submission.destroy
    response = { 'status': 'deleted' }
    render status: :ok, json: response
  end

  private

  def assign_curators
    curators = Role.where(name: "curator").first.users.sample(5)
    @submission.assignees = curators
    #TODO: notify curators (email?)
  end

  def answer_params
    return nil if params[:submission].nil?
    params[:submission].permit(:justification, answers: [:question_id, :accepted])
  end

  def submission_params
    return nil if params[:submission].nil?
    params[:submission].permit(:learning_object_id)
  end

  def set_new_submission
    @submission ||= Submission.new
  end

  def set_submission
    @submission ||= Submission.find_by_id(params[:id])
    if @submission.blank?
      render status: :not_found
    end
  end

  def authorize!
    authorize @submission
  end
end
