
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

module TagSearchService
  # DEBUG = true
  FTREE_CACHE_KEY = "ftree".freeze
  PAD = 1
  PADM = 0.05

  def root_dir
    Rails.root.join("tmp").to_s
  end

  def file_path
    Rails.root.join(root_dir, "tags").to_s
  end

  def parse_ftree_cache
    Rails.cache.fetch(FTREE_CACHE_KEY) do
      parse_ftree(file_path)
    end
  end

  def search(tag, limit = -1)
    # Create clusters from ftree
    ftree = parse_ftree_cache()
    tag_cluster_hash = ftree[:tag_cluster_hash]

    relevant_los = find_relevant_results(tag.id, tag_cluster_hash, limit)

    search_los = LearningObject.search tag.name, limit: limit, explain:true

    merged_los = merge(search_los, relevant_los)

    return merged_los.map { |lc| lc[0] }
  end

  def merge(search_los, relevant_los)
    merged_los = []

    max = search_los.first.search_hit['_score']
    min = search_los[search_los.size-1].search_hit['_score']
    max_boost = 0
    response = search_los.response['hits']['hits']
    search_los.each_with_index do |slo, i|
      detail = response[i]['_explanation']['details'][0]
      while detail['description'] != "boost"
        detail = detail['details'][0]
      end
      boost = detail['value']
      max_boost = boost if boost > max_boost

      slo.search_hit['_score'] = boost*(slo.search_hit['_score']*(1+PADM)-min)/(max-min)
    end

    max = relevant_los.first[1]
    min = relevant_los.last[1]

    relevant_los.each do |rlc|
      rlc[1] = max_boost*(rlc[1]*(1+PADM)-min)/(max-min)
    end if max != 0

    search_los.each do |slo|
      relevant_los.each_with_index do |rlc, index|
        if slo.id == rlc[0].id
          slo.search_hit['_score'] = slo.search_hit['_score'] + rlc[1]
          relevant_los.delete_at(index)
        end
      end
      merged_los << [slo, slo.search_hit['_score']]
    end

    merged_los.push(*relevant_los)
    # highest ranks first
    merged_los = merged_los.sort_by { |lc| lc[1]*-1 }
    return merged_los
  end

  def find_relevant_results(tagId, tag_cluster_hash, limit)
    los_ranked = {}

    close_tags = ranked_close_tags(tagId, tag_cluster_hash)

    # Uncomment the next line if you want to sort by global tag frequency
    # freq_cluster = cluster_frequency(close_tags)

    close_tags.each do |ct|
      tag = Tag.find(ct[:id])
      tag.taggings.where(taggable_type: "LearningObject").each do |tagging|
        lo = tagging.taggable
        if los_ranked[lo].nil?
          # Uncomment the next line if you want to sort by local tag frequency
          # los_ranked[lo.id] = relevance_frequency_rank(lo, close_tags)

          # Uncomment the next line if you want to sort by global tag frequency
          # los_ranked[lo.id] = relevance_frequency_rank_global(lo, close_tags, freq_cluster)

          # Uncomment the next line if you want to sort by tag cluster rank
          los_ranked[lo] = relevance_raw_rank(lo, close_tags)
        end
      end
    end

    # highest ranks first
    los_ranked = los_ranked.sort_by { |lo, rank| rank*-1 }
    return los_ranked.first(limit) if limit > 0
    return los_ranked
  end

  # ranking #
  def ranked_close_tags(tagId, tag_cluster_hash)
    close_tags = []

    tag_cluster_hash[tagId][:parent][:childs].each do |t|
      # calculate logarithmic distance between tag flows
      # lower value, closer, more relevant
      # the tag you are searching for will be at distance 0 of itself
      lg_dist = ( Math.log2(tag_cluster_hash[tagId][:flow]) - Math.log2(t[:flow]) ).abs

      close_tags << { id: t[:id], rank: lg_dist}
    end

    return normalize_complement_close(close_tags)
  end

  # normalizes and complements, 0 distance will be 1,
  # max dist will be closest to 0
  def normalize_complement_close(tags)
    max = 0

    # find max rank
    tags.each do |t|
      max = t[:rank] if t[:rank] > max
    end

    # normalize, min will always be 0
    tags.each do |t|
      # increase max by $PAD so its rank isn't 0
      t[:rank] = 1 - (t[:rank]/(max*(1 + PAD)))
    end if max != 0

    return tags
  end

  def relevance_frequency_rank(lo, close_tags)
    itf_sum = 0

    wdf = 0
    wdf = 1/(Math.log2(lo.tags.size)+1) if lo.tags.size != 0

    lo.tags.each do |t|
      close_tags.each do |tag|
        if tag[:id] == t.id
          itf_sum += tag[:rank]*(Math.log2(Tag.all.size/t.taggings.size)+1)
        end
      end
    end

    return wdf*itf_sum
  end

  # returns the sum of how many times each tag in cluster appears in space
  def cluster_frequency(cluster)
    freq_cluster = 0
    cluster.each do |t|
      freq_cluster += Tag.find(t[:id]).taggings.size
    end

    return freq_cluster
  end

  def relevance_frequency_rank_global(lo, close_tags, freq_cluster)
    # for each tag in LO that is in the cluster, accumulate it's rank
    rank_sum = 1
    lo.tags.each do |t|
      close_tags.each do |tag|
        if tag[:id] == t.id
          rank_sum += tag[:rank]
        end
      end
    end

    wdf = 0
    wdf = (Math.log2(rank_sum)/(Math.log2(lo.tags.size)+1)) if lo.tags.size != 0

    itf = Math.log2(Tag.all.size/freq_cluster)+1

    return wdf*itf
  end

  # returns the rank sum of the tags in the LO
  def relevance_raw_rank(lo, close_tags)
    # for each tag in LO that is in the cluster, accumulate it's rank
    rank_sum = 0
    lo.tags.each do |t|
      close_tags.each do |tag|
        if tag[:id] == t.id
          rank_sum += tag[:rank]
        end
      end
    end

    return rank_sum
  end

  def parse_ftree(path)
    # parse .ftree into a 'tree', leafs are tags with flow number, branches are the clustes
    # create tags list, tags[tagId] == the tag leaf inside the cluster tree

    root = {childs: [], parent: nil}
    tag_cluster_hash = {}
    tag_clusters = []

    File.open(path+".ftree", "r") do |f|
      f.gets
      f.gets

      while line = f.gets
        break if !line.include? ':'

        tmp = line.split(' ')
        # tmp[0] format like: 2:7:5:4, 2 an 7 have clusters as childs, 5 has leafs, 4 is one of them (tag)
        # get id of each level of the tree
        ftree = tmp[0].split(':')[0..-2]
        # last number of the sequence is the leaf Id
        leafId = tmp[0].split(':')[-1].to_i

        # last number on the line is the tag Id
        tagId = tmp[-1].to_i
        # second number on the line
        flow = tmp[1].to_f
        # between the third and second to last is where the name lies
        name = tmp[2..-2].join(' ')[1..-2]

        # iterate through the levels of the tree
        it = root # restart at the root
        ftree.each do |clusterId|
          clusterId = clusterId.to_i - 1 # on the file they start at 1, we want 0
          if it[:childs][clusterId].nil? # if this id doesn't exist, create it as child of 'it'
            it[:childs][clusterId] = {childs: [], parent: nil}
            it[:childs][clusterId][:parent] = it
          end
          # go down a level
          it = it[:childs][clusterId]
        end
        tag_clusters << it
        # 'it' is the cluster leafId is a child of, so add it
        it[:childs][leafId-1] = {id: tagId, flow: flow, name: name, parent: it}
        # put the leaf on this hash for easy acess by the tagId
        tag_cluster_hash[tagId] = it[:childs][leafId-1]
      end
    end

    return {tag_cluster_hash: tag_cluster_hash, tag_clusters: tag_clusters}
  end

end
