
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'openssl'

namespace :import do
  desc 'import learning objects from dspace'

  task :sync_learning_objects => :environment do
    desc 'Synchronize Learning Objects with DSpace Items'
    OpenSSL::SSL::VERIFY_PEER = OpenSSL::SSL::VERIFY_NONE

    # IDEA: create DSpace collections once dspace_sets are updated
    #       and then search for items based on the collection id
    # TODO: Deal with already existing item/LO
    # TODO: Deal with deleted items
    params = {
      q: "search.resourcetype:2", # AND location.coll:4
      fq: "dc.date.accessioned_dt:[NOW-1DAY TO NOW]",
      wt: "json",
      indent: "true"
    }
    uri = URI(DspaceService.link + "/solr/search/select")

    users = User.joins(:roles).where(roles: {name: "partner"}).where.not(dspace_url: [nil, ""], dspace_handle: [nil, ""])
    users.each do |user|
      user_params = params
      user_params[:q] += " AND handle: #{user.dspace_handle}/*"

      uri.query = URI.encode_www_form(user_params)
      response = JSON.parse(Net::HTTP.get(uri))
      items = []
      response["response"]["docs"].each do |doc|
        item = dspace_client.items.find(
          id: doc["search.resourceid"],
          expand: "metadata,bitstreams,parentCollection"
        )
        if items.is_a? String
          logger.warn "Received a string instead of item: #{item}"
          next
        end
        items << item
      end
      importer = Dspace::LearningObjectImporter.new(items, user)
      importer.import
    end
  end

  task :learning_object, [:log] => [:environment] do |_t, args|
    desc 'Importing Learning Objects from Dspace items'

    args.with_defaults(log: STDOUT)
    logger = Log::DatabaseLogger.new(args.log)
    logger.level = Log::DatabaseLogger::CREATE

    # Quantity of items fetched on each iteration
    limit = 500

    # Start point from where items will be fetched
    offset = 0

    loop do
      logger.info " --> importing items from #{offset} to #{offset + limit}"

      begin
        # Get items from dspace (from offset to offset+limit)
        dspace_items = get_col_items_from_dspace(limit, offset, ['metadata', 'bitstreams', 'parentCollection'])
      rescue => e
        logger.warn 'Database error, going to sleep'
        logger.error e
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if dspace_items.empty?

        # continue if receive an error string
        if dspace_items.is_a? String
          logger.warn "Received a string instead of items: #{dspace_items}"
          next
        end

        # Increment offset, to get new items on next iteration
        offset += limit

        dspace_items.select! do |i|
          i.parent_collection.id == DspaceService::BD_INT_OBJETOS_COLLECTION
        end
        next if dspace_items.empty?
        # Iterates through items
        # Verifies if item is already on repository
        # Initializes new LearningObjects
        # and saves them on LearningObjects's repository
        importer = Dspace::LearningObjectImporter.new(dspace_items)
        importer.import
      end
    end
    logger.close
  end

  private

  def dspace_client
    @client ||= DspaceService.create_client
  end

  def get_col_items_from_dspace(limit, offset, expand = [])
    dspace_client.items.all(
      expand: expand.join(','),
      limit: limit,
      offset: offset
    )
  end
end
