
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'acceptance_helpers'
require 'shared/examples'
require 'shared/contexts'

resource 'Collections' do
  header 'Accept', 'application/json'
  header 'Content-Type', 'application/json'

  explanation "Collection is a group of educational content or another collection, which an user can show, create, update and destroy."

  before { 12.times { create(:collection, privacy: 'public' ); create(:learning_object) } } 

  let(:collections) { Collection.all }
  let(:learning_objects) { LearningObject.all }
  let(:_subjects) { Subject.all }
  let(:_educational_stages) { EducationalStage.all }

  post '/v1/collections' do
    include_context "authenticate_user"

    parameter :name, 'The name of the collection', scope: :collection
    parameter :description, 'The description of educational content of the collection', scope: :collection
    parameter :owner_id, 'The id of the collection’s owner', scope: :collection
    parameter :owner_type, 'Represents a type of the collections owner [User, Institution]', scope: :collection
    parameter :privacy, 'Represents if collections is public, or not. [public, private]', scope: :collection
    parameter :items, 'Items belongs to the collection [LearningObject, Collection]', scope: :collection
    parameter :tags, 'Array with tags', scope: :collection
    parameter :subjects, 'Array with subjects', scope: :collection
    parameter :educational_stages, 'Array with educational stages', scope: :collection

    let(:name) { Faker::Lorem.word }
    let(:description) { Faker::Lorem.sentence }
    let(:owner_id) { @user.id }
    let(:owner_type) { 'User' }
    let(:privacy) { 'public' }
    let(:items) { [ { id: learning_objects.first.id, type: "LearningObject", position: 0  } ] }
    let(:tags) { [ { name: Faker::Lorem.word} ] }
    let(:educational_stages) { [_educational_stages.first.id, _educational_stages.second.id] }
    let(:subjects) { [_subjects.first.id, _subjects.second.id] }
    let(:raw_post) { params.to_json }

    example 'Creating a collection' do
      do_request
      expect(status).to eq(201)
    end
  end

   get '/v1/collections' do
     parameter :limit, 'Limit of collections'
     parameter :offset, 'Offset of collections'

     let(:limit) { 12 }
     let(:offset) { 0 }

     example_request 'Get a list of collections' do
       expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(Collection.limit(limit).offset(offset).pluck(:id).sort)
       expect(status).to eq(200)
     end
   end

  get '/v1/collections/:id' do
    let(:id) { collections.first.id }

    example 'Get a collection' do
      do_request
      expect(path).to eq("/v1/collections/#{id}") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(Collection.find(id)))
      expect(status).to eq(200)
    end
  end

  delete '/v1/collections/:id' do
    include_context "authenticate_user"

    let(:id) { @collection.id }

    before do
      @collection = create(:collection, owner: @user)
    end

    example 'Destroying a collection' do
      do_request
      expect(status).to eq(200)
    end

  end

  post '/v1/collections/:id/items' do
    include_context "authenticate_user"

    parameter :items, 'Items belongs to the collection', scope: :collection

    let(:id) { @collection.id }
    let(:items) { [ { id: learning_objects.first.id, type: "LearningObject", position: "4" } ] }
    let(:raw_post) { params.to_json }

    before do
      @collection = create(:collection, owner: @user)
    end

    example 'Add items to collection' do
      do_request
      expect(status).to eq(200)
    end
  end

  delete '/v1/collections/:id/items' do
    include_context "authenticate_user"

    parameter :items, 'Items belongs to the collection', scope: :collection

    let(:id) { @collection.id }
    let(:items) { [ { id: @collection_item.id } ] }
    let(:raw_post) { params.to_json }

    before do
      @collection = create(:collection, owner: @user)
      @collection_item = create(:collection_item, collection: @collection)
    end

    example 'Remove items from collection' do
      do_request
      expect(status).to eq(200)
    end

  end


  put '/v1/collections/:id/like' do
    include_context "authenticate_user"

    let(:id) { collections.first.id }

    example 'Liking a collection' do
      likes = collections.first.likes.count
      do_request
      expect(path).to eq("/v1/collections/#{id}/like")
      expect(JSON.parse(response_body)['count']).to eq(likes + 1)
      expect(status).to eq(201)
    end
  end

  put '/v1/collections/:id/like' do
    include_context "authenticate_user"

    let(:id) { collections.first.id }

    before do
      create(:like, user: @user, likeable: collections.first)
    end

    example 'Unliking a collection' do
      likes = collections.first.likes.count
      do_request
      expect(path).to eq("/v1/collections/#{id}/like")
      expect(JSON.parse(response_body)['count']).to eq(likes - 1)
      expect(status).to eq(200)
    end
  end

  get '/v1/collections/:id/versions' do
    include_context "authenticate_user"

    before do
      @collection = create(:collection)
      @collection.update(name: Faker::Name.name)
    end

    let(:id) { @collection.id }

    example 'Getting all collection versions' do
      do_request
      expect(path).to eq("/v1/collections/#{id}/versions") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(@collection.versions))
      expect(status).to eq(200)
    end
  end

  put '/v1/collections/:id' do
    include_context "authenticate_user"

    let(:id) { @collection.id }
    let(:name) { Faker::Name.name }
    let(:raw_post) { params.to_json }

    parameter :id, 'The id of the collection'
    parameter :name, 'The collection name'
    parameter :Collection, 'The collection to be updated'

    before do
      @collection = create(:collection, owner: @user)
    end

    example 'Updating a collection' do
      do_request
      expect(path).to eq("/v1/collections/#{id}") # `:id` is replaced with the value of `id`
      #TODO: compair responses
      expect(status).to eq(200)
    end
  end

end
